/*
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/


#pragma once

#include <string>
#include <tuple>
#include <unordered_map>
#include <vector>

namespace ppds {

    enum class NodeType {
        TableScan,
        Projection,
        Predicate,
        EquiJoin,
        GroupBy
    };

    /*
     * Node parameters
     *
     * TableScan
     * "file_path": The path to the table file (e.g., "/path/to/file.tbl")
     *
     * Projection
     * "column_ids": The columns to project (e.g., "0,1,3")
     *
     * Predicate
     * "column_id": The column to apply the predicate to (e.g., "1")
     * "condition": Either "=", "!=", "<", "<=", ">", or ">="
     * "literal": The constant to compare to (e.g., "15" or "random")
     *
     * EquiJoin
     * "left_column_id": The left child's join column (e.g., "3")
     * "right_column_id": The the right child's join column (e.g., "0")
     *
     * GroupBy
     * ° "group_by_column_id": The column to group (e.g., "2")
     * "aggregate_column_id": The column to aggregate
     * "aggregate_type": Either "avg", "max", or "min"
     *
     * ° = optional
     */
    struct PlanNode {
        PlanNode(NodeType type_p, std::unordered_map<std::string, std::string> parameters_p,
                        std::vector<PlanNode> children_p) :
                        type(type_p), parameters(std::move(parameters_p)), children(std::move(children_p)) {}
        const NodeType type;
        const std::unordered_map<std::string, std::string> parameters;
        const std::vector<PlanNode> children;
    };

    enum class DataType {
        Integer,
        Float,
        String
    };

    class Record {
    public:
        virtual void Initialize(std::shared_ptr<std::vector<DataType>> types_p) {
            schema = std::move(types_p);
            data.resize(schema->size());
            is_valid = true;
        }

        virtual inline std::variant<int32_t, float, std::string>& GetValue(uint16_t index) {
            return data[index];
        }

        virtual inline void SetValue(uint16_t index, std::variant<int32_t, float, std::string> value) {
            data[index] = std::move(value);
        }

        virtual inline uint16_t GetColumnCount() {
            return schema->size();
        }

        virtual inline DataType GetColumnType(uint16_t index) {
            return (*schema)[index];
        }

        virtual inline std::shared_ptr<std::vector<DataType>> GetColumnTypes() {
            return schema;
        }

        virtual void Clear() {
            data.clear();
            schema = nullptr;
            is_valid = false;
        }

        virtual inline bool IsValid() {
            return is_valid;
        }

    protected:
        std::vector<std::variant<int32_t, float, std::string>> data;
        std::shared_ptr<std::vector<DataType>> schema;
        bool is_valid = false;
    };

    class ONCIteratorState{};

    class ONCIterator {
    public:
        virtual void Open() = 0;
        virtual void Next(Record& result) = 0;
        virtual void Close() = 0;
    };

}
